==> squabble-1.0.0rc4.tar.gz: squabble-1.0.0rc4/squabble/message.py <==
from squabble import SquabbleException, logger


class DuplicateMessageCodeException(SquabbleException):
    def __init__(self, dupe):
        original = Registry.by_code(dupe.CODE)
        message = 'Message %s has the same code as %s' % (dupe, original)

        super().__init__(message)


class Registry:
    """
    Singleton which maps message code values to classes.

    >>> class MyMessage(Message):
    ...     '''My example message.'''
    ...     TEMPLATE = '...'
    ...     CODE = 5678
    >>> cls = Registry.by_code(5678)
    >>> cls.explain()
    'My example message.'
    >>> cls is MyMessage
    True

    Duplicate codes are not allowed, and will throw an exception.

    >>> class MyDuplicateMessage(Message):
    ...     CODE = 5678
    Traceback (most recent call last):
      ...
    squabble.message.DuplicateMessageCodeException: ...
    """

    _MAP = {}
    _CODE_COUNTER = 9000

    @classmethod
    def register(cls, msg):
        """
        Add ``msg`` to the registry, and assign a ``CODE`` value if not
        explicitly specified.
        """
        if not hasattr(msg, 'CODE'):
            setattr(msg, 'CODE', cls._next_code())
            logger.info('assigning code %s to %s', msg.CODE, msg)

        # Don't allow duplicates
        if msg.CODE in cls._MAP:
            raise DuplicateMessageCodeException(msg)

        cls._MAP[msg.CODE] = msg

    @classmethod
    def by_code(cls, code):
        """
        Return the :class:`squabble.message.Message` class identified by
        ``code``, raising a :class:`KeyError` if it doesn't exist.
        """
        return cls._MAP[code]

    @classmethod
    def _next_code(cls):
        cls._CODE_COUNTER += 1
        return cls._CODE_COUNTER


class Message:
    """
    Messages represent specific issues identified by a lint rule.

    Each class that inherits from ``Message`` should have a docstring
    which explains the reasoning and context of the message, as well
    as a class member variable named ``TEMPLATE``, which is used to
    display a brief explanation on the command line.

    Messages may also have a ``CODE`` class member, which is used to
    identify the message. The actual value doesn't matter much, as
    long as it is unique among all the loaded ``Message`` s. If no
    ``CODE`` is defined, one will be assigned.

    >>> class TooManyColumns(Message):
    ...    '''
    ...    This may indicate poor design, consider multiple tables instead.
    ...    '''
    ...    TEMPLATE = 'table "{table}" has > {limit} columns'
    ...    CODE = 1234
    >>> message = TooManyColumns(table='foo', limit=30)
    >>> message.format()
    'table "foo" has > 30 columns'
    >>> message.explain()
    'This may indicate poor design, consider multiple tables instead.'
    """
    TEMPLATE = None

    def __init__(self, **kwargs):
        self.kwargs = kwargs

    def __init_subclass__(cls, **kwargs):
        """Assign unique (locally, not globally) message codes."""
        super().__init_subclass__(**kwargs)
        Registry.register(cls)

    def format(self):
        return self.TEMPLATE.format(**self.kwargs)

    @classmethod
    def explain(cls):
        """
        Provide more context around this message.

        The purpose of this function is to explain to users _why_ the
        message was raised, and what they can do to resolve the issue.

        The base implementation will simply return the docstring for
        the class, but this can be overridden if more specialized
        behavior is necessary.

        >>> class NoDocString(Message): pass
        >>> NoDocString().explain() is None
        True
        """
        if not cls.__doc__:
            return None

        return cls.__doc__.strip()

    def asdict(self):
        return {
            'message_id': self.__class__.__name__,
            'message_text': self.format(),
            'message_template': self.TEMPLATE,
            'message_params': self.kwargs,
            'message_code': self.CODE
        }

==> squabble-1.1.0rc0.tar.gz: squabble-1.1.0rc0/squabble/message.py <==
import inspect
import logging

from squabble import SquabbleException
from squabble.util import strip_rst_directives


logger = logging.getLogger(__name__)


class DuplicateMessageCodeException(SquabbleException):
    def __init__(self, dupe):
        original = Registry.by_code(dupe.CODE)
        message = 'Message %s has the same code as %s' % (dupe, original)

        super().__init__(message)


class Registry:
    """
    Singleton which maps message code values to classes.

    >>> class MyMessage(Message):
    ...     '''My example message.'''
    ...     TEMPLATE = '...'
    ...     CODE = 5678
    >>> cls = Registry.by_code(5678)
    >>> cls.explain()
    'My example message.'
    >>> cls is MyMessage
    True

    Duplicate codes are not allowed, and will throw an exception.

    >>> class MyDuplicateMessage(Message):
    ...     CODE = 5678
    Traceback (most recent call last):
      ...
    squabble.message.DuplicateMessageCodeException: ...
    """

    _MAP = {}
    _CODE_COUNTER = 9000

    @classmethod
    def register(cls, msg):
        """
        Add ``msg`` to the registry, and assign a ``CODE`` value if not
        explicitly specified.
        """
        if not hasattr(msg, 'CODE'):
            setattr(msg, 'CODE', cls._next_code())
            logger.info('assigning code %s to %s', msg.CODE, msg)

        # Don't allow duplicates
        if msg.CODE in cls._MAP:
            raise DuplicateMessageCodeException(msg)

        cls._MAP[msg.CODE] = msg

    @classmethod
    def by_code(cls, code):
        """
        Return the :class:`squabble.message.Message` class identified by
        ``code``, raising a :class:`KeyError` if it doesn't exist.
        """
        return cls._MAP[code]

    @classmethod
    def _next_code(cls):
        cls._CODE_COUNTER += 1
        return cls._CODE_COUNTER


class Message:
    """
    Messages represent specific issues identified by a lint rule.

    Each class that inherits from ``Message`` should have a docstring
    which explains the reasoning and context of the message, as well
    as a class member variable named ``TEMPLATE``, which is used to
    display a brief explanation on the command line.

    Messages may also have a ``CODE`` class member, which is used to
    identify the message. The actual value doesn't matter much, as
    long as it is unique among all the loaded ``Message`` s. If no
    ``CODE`` is defined, one will be assigned.

    >>> class TooManyColumns(Message):
    ...    '''
    ...    This may indicate poor design, consider multiple tables instead.
    ...    '''
    ...    TEMPLATE = 'table "{table}" has > {limit} columns'
    ...    CODE = 1234
    >>> message = TooManyColumns(table='foo', limit=30)
    >>> message.format()
    'table "foo" has > 30 columns'
    >>> message.explain()
    'This may indicate poor design, consider multiple tables instead.'
    """
    TEMPLATE = None

    def __init__(self, **kwargs):
        self.kwargs = kwargs

    def __init_subclass__(cls, **kwargs):
        """Assign unique (locally, not globally) message codes."""
        super().__init_subclass__(**kwargs)
        Registry.register(cls)

    def format(self):
        return self.TEMPLATE.format(**self.kwargs)

    @classmethod
    def explain(cls):
        """
        Provide more context around this message.

        The purpose of this function is to explain to users _why_ the
        message was raised, and what they can do to resolve the issue.

        The base implementation will simply return the docstring (with
        some common RST-isms stripped out) for the class, but this can
        be overridden if more specialized behavior is necessary.

        >>> class NoDocString(Message): pass
        >>> NoDocString().explain() is None
        True

        >>> class RstMessage(Message):
        ...    '''
        ...    Don't do this:
        ...    .. code-block:: sql
        ...      SELECT * FROM big_table;
        ...    '''
        >>> RstMessage().explain()
        "Don't do this:\\n  SELECT * FROM big_table;"
        """
        if not cls.__doc__:
            return None

        # Remove the leading indentation on the docstring
        doc = inspect.cleandoc(cls.__doc__)

        # replace ``.. code-block:: lang``
        return strip_rst_directives(doc)

    def asdict(self):
        return {
            'message_id': self.__class__.__name__,
            'message_text': self.format(),
            'message_template': self.TEMPLATE,
            'message_params': self.kwargs,
            'message_code': self.CODE
        }
